@@ -39,7 +39,6 @@
 #include "gdbusintrospection.h"
 #include "gdbusmethodinvocation.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION: gapplication
@@ -131,11 +130,12 @@
  * application instance when a second instance fails to take the bus name.
  * @arguments contains the commandline arguments given to the second instance
  * and @data contains platform-specific additional data.
- * 
- * On all platforms, @data is guaranteed to have a key "cwd" of type
- * signature "ay" which contains the working directory of the invoked
- * executable.
- * 
+ *
+ * On all platforms, @data will have a key "cwd" of type signature
+ * "ay" which contains the working directory of the invoked
+ * executable; this data is defined to be in the default GLib
+ * filesystem encoding for the platform.  See g_filename_to_utf8().
+ *
  * </para>
  * <para>
  * The <methodname>InvokeAction</methodname> function can be called to
@@ -319,24 +319,35 @@ g_application_default_run (GApplication *application)
 }
 
 static GVariant *
-variant_from_argv (int    argc,
-		   char **argv)
+append_cwd_to_platform_data (GVariant *platform_data)
 {
   GVariantBuilder builder;
-  int i;
+  gchar *cwd;
+  GVariant *result;
 
-  g_variant_builder_init (&builder, G_VARIANT_TYPE ("aay"));
+  cwd = g_get_current_dir ();
 
-  for (i = 1; i < argc; i++)
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
+  if (cwd)
+    g_variant_builder_add (&builder, "{sv}",
+			   "cwd",
+			   g_variant_new_bytestring (cwd));
+  g_free (cwd);
+
+  if (platform_data)
     {
-      guint8 *argv_bytes;
+      GVariantIter iter;
+      GVariant *item;
 
-      argv_bytes = (guint8*) argv[i];
-      g_variant_builder_add_value (&builder,
-				   g_variant_new_byte_array (argv_bytes, -1));
+      g_variant_iter_init (&iter, platform_data);
+      while (g_variant_iter_next (&iter, "@{sv}", &item))
+	{
+	  g_variant_builder_add_value (&builder, item);
+	  g_variant_unref (item);
+	}
     }
-  
-  return g_variant_builder_end (&builder);
+  result = g_variant_builder_end (&builder);
+  return result;
 }
 
 static gboolean
@@ -408,10 +419,11 @@ g_application_action_free (gpointer data)
  * GApplication::activate_with_data signal will be emitted in the
  * remote process, with the data from @argv and other
  * platform-specific data available.  Subsequently the
- * #GApplication:default-exit property will be evaluated.  If it's
+ * #GApplication:default-quit property will be evaluated.  If it's
  * %TRUE, then the current process will terminate.  If %FALSE, then
  * the application remains in the #GApplication:is-remote state, and
- * you can e.g. call g_application_invoke_action().
+ * you can e.g. call g_application_invoke_action(). Note that proxy
+ * instances should not call g_application_add_action().
  *
  * This function may do synchronous I/O to obtain unique ownership
  * of the application id, and will block the calling thread in this
@@ -434,6 +446,7 @@ g_application_new (const gchar *appid,
 		   int          argc,
 		   char       **argv)
 {
+  const gchar * const *args = (const gchar **) argv;
   GObject *app;
   GError *error = NULL;
   GVariant *argv_variant;
@@ -442,7 +455,7 @@ g_application_new (const gchar *appid,
 
   g_return_val_if_fail (appid != NULL, NULL);
   
-  argv_variant = variant_from_argv (argc, argv);
+  argv_variant = g_variant_new_bytestring_array (args, argc);
   
   app = g_initable_new (G_TYPE_APPLICATION, 
 			NULL,
@@ -480,13 +493,14 @@ g_application_try_new (const gchar *appid,
 		       char       **argv,
 		       GError     **error)
 {
+  const gchar * const *args = (const gchar **) argv;
   GVariant *argv_variant;
 
   g_type_init ();
 
   g_return_val_if_fail (appid != NULL, NULL);
   
-  argv_variant = variant_from_argv (argc, argv);
+  argv_variant = g_variant_new_bytestring_array (args, argc);
   
   return G_APPLICATION (g_initable_new (G_TYPE_APPLICATION, 
 					NULL,
@@ -517,13 +531,14 @@ g_application_unregistered_try_new (const gchar *appid,
 				    char       **argv,
 				    GError     **error)
 {
+  const gchar * const *args = (const gchar **) argv;
   GVariant *argv_variant;
 
   g_type_init ();
 
   g_return_val_if_fail (appid != NULL, NULL);
   
-  argv_variant = variant_from_argv (argc, argv);
+  argv_variant = g_variant_new_bytestring_array (args, argc);
   
   return G_APPLICATION (g_initable_new (G_TYPE_APPLICATION, 
 					NULL,
@@ -976,6 +991,7 @@ g_application_init (GApplication *app)
   app->priv->default_quit = TRUE;
   app->priv->do_register = TRUE;
   app->priv->is_remote = TRUE;
+  app->priv->platform_data = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
 }
 
 static void
@@ -1045,7 +1061,12 @@ g_application_set_property (GObject      *object,
       break;
 
     case PROP_PLATFORM_DATA:
-      app->priv->platform_data = g_value_dup_variant (value);
+      {
+	GVariant *platform_data = g_value_get_variant (value);
+	if (app->priv->platform_data)
+	  g_variant_unref (app->priv->platform_data);
+	app->priv->platform_data = g_variant_ref_sink (append_cwd_to_platform_data (platform_data));
+      }
       break;
 
     default:
@@ -1301,6 +1322,3 @@ g_application_class_init (GApplicationClass *klass)
 							 G_PARAM_CONSTRUCT_ONLY |
 							 G_PARAM_STATIC_STRINGS));
 }
-
-#define __G_APPLICATION_C__
-#include "gioaliasdef.c"
