@@ -31,7 +31,6 @@
 #include "gdbusconnection.h"
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusnameowning
@@ -450,7 +449,7 @@ connection_get_cb (GObject      *source_object,
 
 /**
  * g_bus_own_name_on_connection:
- * @connection: A #GDBusConnection that is not closed.
+ * @connection: A #GDBusConnection.
  * @name: The well-known name to own.
  * @flags: A set of flags from the #GBusNameOwnerFlags enumeration.
  * @name_acquired_handler: Handler to invoke when @name is acquired or %NULL.
@@ -478,7 +477,6 @@ g_bus_own_name_on_connection (GDBusConnection          *connection,
   Client *client;
 
   g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), 0);
-  g_return_val_if_fail (!g_dbus_connection_is_closed (connection), 0);
   g_return_val_if_fail (g_dbus_is_name (name) && !g_dbus_is_unique_name (name), 0);
 
   G_LOCK (lock);
@@ -737,13 +735,22 @@ g_bus_own_name_with_closures (GBusType                  bus_type,
   data = g_new0 (OwnNameData, 1);
 
   if (bus_acquired_closure != NULL)
-    data->bus_acquired_closure = g_closure_ref (bus_acquired_closure);
+    {
+      data->bus_acquired_closure = g_closure_ref (bus_acquired_closure);
+      g_closure_sink (bus_acquired_closure);
+    }
 
   if (name_acquired_closure != NULL)
-    data->name_acquired_closure = g_closure_ref (name_acquired_closure);
+    {
+      data->name_acquired_closure = g_closure_ref (name_acquired_closure);
+      g_closure_sink (name_acquired_closure);
+    }
 
   if (name_lost_closure != NULL)
-    data->name_lost_closure = g_closure_ref (name_lost_closure);
+    {
+      data->name_lost_closure = g_closure_ref (name_lost_closure);
+      g_closure_sink (name_lost_closure);
+    }
 
   return g_bus_own_name (bus_type,
           name,
@@ -757,7 +764,7 @@ g_bus_own_name_with_closures (GBusType                  bus_type,
 
 /**
  * g_bus_own_name_on_connection_with_closures:
- * @connection: A #GDBusConnection that is not closed.
+ * @connection: A #GDBusConnection.
  * @name: The well-known name to own.
  * @flags: A set of flags from the #GBusNameOwnerFlags enumeration.
  * @name_acquired_closure: (allow-none): #GClosure to invoke when @name is
@@ -787,10 +794,16 @@ g_bus_own_name_on_connection_with_closures (GDBusConnection          *connection
   data = g_new0 (OwnNameData, 1);
 
   if (name_acquired_closure != NULL)
-    data->name_acquired_closure = g_closure_ref (name_acquired_closure);
+    {
+      data->name_acquired_closure = g_closure_ref (name_acquired_closure);
+      g_closure_sink (name_acquired_closure);
+    }
 
   if (name_lost_closure != NULL)
-    data->name_lost_closure = g_closure_ref (name_lost_closure);
+    {
+      data->name_lost_closure = g_closure_ref (name_lost_closure);
+      g_closure_sink (name_lost_closure);
+    }
 
   return g_bus_own_name_on_connection (connection,
           name,
@@ -894,6 +907,3 @@ g_bus_unown_name (guint owner_id)
       client_unref (client);
     }
 }
-
-#define __G_DBUS_NAME_OWNING_C__
-#include "gioaliasdef.c"
