@@ -32,7 +32,6 @@
 #include "gdbusconnection.h"
 
 #include "glibintl.h"
-#include "gioalias.h"
 
 /**
  * SECTION:gdbusnamewatching
@@ -590,7 +589,7 @@ g_bus_watch_name (GBusType                  bus_type,
 
 /**
  * g_bus_watch_name_on_connection:
- * @connection: A #GDBusConnection that is not closed.
+ * @connection: A #GDBusConnection.
  * @name: The name (well-known or unique) to watch.
  * @flags: Flags from the #GBusNameWatcherFlags enumeration.
  * @name_appeared_handler: Handler to invoke when @name is known to exist or %NULL.
@@ -761,7 +760,7 @@ g_bus_watch_name_with_closures (GBusType                 bus_type,
 
 /**
  * g_bus_watch_name_on_connection_with_closures:
- * @connection: A #GDBusConnection that is not closed.
+ * @connection: A #GDBusConnection.
  * @name: The name (well-known or unique) to watch.
  * @flags: Flags from the #GBusNameWatcherFlags enumeration.
  * @name_appeared_closure: (allow-none): #GClosure to invoke when @name is known
@@ -849,6 +848,3 @@ g_bus_unwatch_name (guint watcher_id)
       client_unref (client);
     }
 }
-
-#define __G_DBUS_NAME_WATCHING_C__
-#include "gioaliasdef.c"
