@@ -26,7 +26,6 @@
 #include "goutputstream.h"
 #include "glibintl.h"
 
-#include "gioalias.h"
 
 /**
  * SECTION:gfilteroutputstream
@@ -307,6 +306,3 @@ g_filter_output_stream_close (GOutputStream  *stream,
 
   return res;
 }
-
-#define __G_FILTER_OUTPUT_STREAM_C__
-#include "gioaliasdef.c"
