@@ -35,14 +35,32 @@
 #include "strinfo.c"
 
 /* Handling of <enum> {{{1 */
-typedef GString EnumState;
+typedef struct
+{
+  GString *strinfo;
+
+  gboolean is_flags;
+} EnumState;
 
 static void
 enum_state_free (gpointer data)
 {
   EnumState *state = data;
 
-  g_string_free (state, TRUE);
+  g_string_free (state->strinfo, TRUE);
+  g_slice_free (EnumState, state);
+}
+
+EnumState *
+enum_state_new (gboolean is_flags)
+{
+  EnumState *state;
+
+  state = g_slice_new (EnumState);
+  state->strinfo = g_string_new (NULL);
+  state->is_flags = is_flags;
+
+  return state;
 }
 
 static void
@@ -58,12 +76,14 @@ enum_state_add_value (EnumState    *state,
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "enum nick must be a minimum of 2 characters");
+                   "nick must be a minimum of 2 characters");
       return;
     }
 
   value = g_ascii_strtoll (valuestr, &end, 0);
-  if (*end || value > G_MAXINT32 || value < G_MININT32)
+  if (*end || state->is_flags ?
+                (value > G_MAXUINT32 || value < 0) :
+                (value > G_MAXINT32 || value < G_MININT32))
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
@@ -71,15 +91,38 @@ enum_state_add_value (EnumState    *state,
       return;
     }
 
-  if (strinfo_builder_contains (state, nick))
+  if (strinfo_builder_contains (state->strinfo, nick))
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<value nick='%s'> already specified", nick);
+                   "<value nick='%s'/> already specified", nick);
       return;
     }
 
-  strinfo_builder_append_item (state, nick, value);
+  if (strinfo_builder_contains_value (state->strinfo, value))
+    {
+      g_set_error (error, G_MARKUP_ERROR,
+                   G_MARKUP_ERROR_INVALID_CONTENT,
+                   "value='%s' already specified", valuestr);
+      return;
+    }
+
+  if (state->is_flags && (value & (value - 1)))
+    {
+      g_set_error (error, G_MARKUP_ERROR,
+                   G_MARKUP_ERROR_INVALID_CONTENT,
+                   "flags values must have at most 1 bit set");
+      return;
+    }
+
+  /* Since we reject exact duplicates of value='' and we only allow one
+   * bit to be set, it's not possible to have overlaps.
+   *
+   * If we loosen the one-bit-set restriction we need an overlap check.
+   */
+
+
+  strinfo_builder_append_item (state->strinfo, nick, value);
 }
 
 static void
@@ -91,7 +134,7 @@ enum_state_end (EnumState **state_ptr,
   state = *state_ptr;
   *state_ptr = NULL;
 
-  if (state->len == 0)
+  if (state->strinfo->len == 0)
     g_set_error_literal (error,
                          G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
                          "<enum> must contain at least one <value>");
@@ -116,12 +159,14 @@ typedef struct
 
   GString      *strinfo;
   gboolean      is_enum;
+  gboolean      is_flags;
 
   GVariant     *minimum;
   GVariant     *maximum;
 
   gboolean      has_choices;
   gboolean      has_aliases;
+  gboolean      is_override;
 
   gboolean      checked;
   GVariant     *serialised;
@@ -130,22 +175,50 @@ typedef struct
 static KeyState *
 key_state_new (const gchar *type_string,
                const gchar *gettext_domain,
-               EnumState   *enum_data)
+               gboolean     is_enum,
+               gboolean     is_flags,
+               GString     *strinfo)
 {
   KeyState *state;
 
   state = g_slice_new0 (KeyState);
   state->type = g_variant_type_new (type_string);
   state->have_gettext_domain = gettext_domain != NULL;
+  state->is_enum = is_enum;
+  state->is_flags = is_flags;
 
-  if ((state->is_enum = (enum_data != NULL)))
-    state->strinfo = g_string_new_len (enum_data->str, enum_data->len);
+  if (strinfo)
+    state->strinfo = g_string_new_len (strinfo->str, strinfo->len);
   else
     state->strinfo = g_string_new (NULL);
 
   return state;
 }
 
+static KeyState *
+key_state_override (KeyState    *state,
+                    const gchar *gettext_domain)
+{
+  KeyState *copy;
+
+  copy = g_slice_new0 (KeyState);
+  copy->type = g_variant_type_copy (state->type);
+  copy->have_gettext_domain = gettext_domain != NULL;
+  copy->strinfo = g_string_new_len (state->strinfo->str,
+                                    state->strinfo->len);
+  copy->is_enum = state->is_enum;
+  copy->is_flags = state->is_flags;
+  copy->is_override = TRUE;
+
+  if (state->minimum)
+    {
+      copy->minimum = g_variant_ref (state->minimum);
+      copy->maximum = g_variant_ref (state->maximum);
+    }
+
+  return copy;
+}
+
 static KeyState *
 key_state_new_child (const gchar *child_schema)
 {
@@ -201,6 +274,10 @@ key_state_check_range (KeyState  *state,
 {
   if (state->default_value)
     {
+      const gchar *tag;
+
+      tag = state->is_override ? "override" : "default";
+
       if (state->minimum)
         {
           if (g_variant_compare (state->default_value, state->minimum) < 0 ||
@@ -208,8 +285,8 @@ key_state_check_range (KeyState  *state,
             {
               g_set_error (error, G_MARKUP_ERROR,
                            G_MARKUP_ERROR_INVALID_CONTENT,
-                           "<default> is not contained in "
-                           "the specified range");
+                           "<%s> is not contained in "
+                           "the specified range", tag);
             }
         }
 
@@ -218,16 +295,22 @@ key_state_check_range (KeyState  *state,
           if (!is_valid_choices (state->default_value, state->strinfo))
             {
               if (state->is_enum)
-                g_set_error_literal (error, G_MARKUP_ERROR,
-                                     G_MARKUP_ERROR_INVALID_CONTENT,
-                                     "<default> is not a valid member of "
-                                     "the specified enumerated type");
+                g_set_error (error, G_MARKUP_ERROR,
+                             G_MARKUP_ERROR_INVALID_CONTENT,
+                             "<%s> is not a valid member of "
+                             "the specified enumerated type", tag);
+
+              else if (state->is_flags)
+                g_set_error (error, G_MARKUP_ERROR,
+                             G_MARKUP_ERROR_INVALID_CONTENT,
+                             "<%s> contains string not in the "
+                             "specified flags type", tag);
 
               else
-                g_set_error_literal (error, G_MARKUP_ERROR,
-                                     G_MARKUP_ERROR_INVALID_CONTENT,
-                                     "<default> contains string not in "
-                                     "<choices>");
+                g_set_error (error, G_MARKUP_ERROR,
+                             G_MARKUP_ERROR_INVALID_CONTENT,
+                             "<%s> contains string not in "
+                             "<choices>", tag);
             }
         }
     }
@@ -414,11 +497,11 @@ key_state_start_aliases (KeyState  *state,
                          G_MARKUP_ERROR_INVALID_CONTENT,
                          "<aliases> already specified for this key");
 
-  if (!state->is_enum && !state->has_choices)
+  if (!state->is_flags && !state->is_enum && !state->has_choices)
     g_set_error_literal (error, G_MARKUP_ERROR,
                          G_MARKUP_ERROR_INVALID_CONTENT,
                          "<aliases> can only be specified for keys with "
-                         "enumerated types or after <choices>");
+                         "enumerated or flags types or after <choices>");
 }
 
 static void
@@ -554,6 +637,7 @@ key_state_serialise (KeyState *state)
               state->strinfo = NULL;
 
               g_variant_builder_add (&builder, "(y@au)",
+                                     state->is_flags ? 'f' :
                                      state->is_enum ? 'e' : 'c',
                                      array);
             }
@@ -671,23 +755,34 @@ is_valid_keyname (const gchar  *key,
 }
 
 /* Handling of <schema> {{{1 */
-typedef struct
+typedef struct _SchemaState SchemaState;
+struct _SchemaState
 {
-  gchar      *path;
-  gchar      *gettext_domain;
+  SchemaState *extends;
 
-  GHashTable *keys;
-} SchemaState;
+  gchar       *path;
+  gchar       *gettext_domain;
+  gchar       *extends_name;
+  gchar       *list_of;
+
+  GHashTable  *keys;
+};
 
 static SchemaState *
 schema_state_new (const gchar  *path,
-                  const gchar  *gettext_domain)
+                  const gchar  *gettext_domain,
+                  SchemaState  *extends,
+                  const gchar  *extends_name,
+                  const gchar  *list_of)
 {
   SchemaState *state;
 
   state = g_slice_new (SchemaState);
   state->path = g_strdup (path);
   state->gettext_domain = g_strdup (gettext_domain);
+  state->extends = extends;
+  state->extends_name = g_strdup (extends_name);
+  state->list_of = g_strdup (list_of);
   state->keys = g_hash_table_new_full (g_str_hash, g_str_equal,
                                        g_free, key_state_free);
 
@@ -732,14 +827,25 @@ schema_state_add_child (SchemaState  *state,
 static KeyState *
 schema_state_add_key (SchemaState  *state,
                       GHashTable   *enum_table,
+                      GHashTable   *flags_table,
                       const gchar  *name,
                       const gchar  *type_string,
                       const gchar  *enum_type,
+                      const gchar  *flags_type,
                       GError      **error)
 {
-  GString *enum_data;
+  SchemaState *node;
+  GString *strinfo;
   KeyState *key;
 
+  if (state->list_of)
+    {
+      g_set_error_literal (error, G_MARKUP_ERROR,
+                           G_MARKUP_ERROR_INVALID_CONTENT,
+                           "can not add keys to a list-of schema");
+      return NULL;
+    }
+
   if (!is_valid_keyname (name, error))
     return NULL;
 
@@ -751,29 +857,58 @@ schema_state_add_key (SchemaState  *state,
       return NULL;
     }
 
-  if ((type_string == NULL) == (enum_type == NULL))
+  for (node = state; node; node = node->extends)
+    if (node->extends)
+      {
+        KeyState *shadow;
+
+        shadow = g_hash_table_lookup (node->extends->keys, name);
+
+        /* in case of <key> <override> <key> make sure we report the
+         * location of the original <key>, not the <override>.
+         */
+        if (shadow && !shadow->is_override)
+          {
+            g_set_error (error, G_MARKUP_ERROR,
+                         G_MARKUP_ERROR_INVALID_CONTENT,
+                         "<key name='%s'> shadows <key name='%s'> in "
+                         "<schema id='%s'>; use <override> to modify value",
+                         name, name, node->extends_name);
+            return NULL;
+          }
+      }
+
+  if ((type_string != NULL) + (enum_type != NULL) + (flags_type != NULL) != 1)
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_MISSING_ATTRIBUTE,
-                   "exactly one of 'type' or 'enum' must "
+                   "exactly one of 'type', 'enum' or 'flags' must "
                    "be specified as an attribute to <key>");
       return NULL;
     }
 
-  if (enum_type != NULL)
+  if (type_string == NULL) /* flags or enums was specified */
     {
-      enum_data = g_hash_table_lookup (enum_table, enum_type);
+      EnumState *enum_state;
 
-      if (enum_data == NULL)
+      if (enum_type)
+        enum_state = g_hash_table_lookup (enum_table, enum_type);
+      else
+        enum_state = g_hash_table_lookup (flags_table, flags_type);
+
+
+      if (enum_state == NULL)
         {
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
-                       "<enum id='%s'> not (yet) defined.", enum_type);
+                       "<%s id='%s'> not (yet) defined.",
+                       flags_type ? "flags"    : "enum",
+                       flags_type ? flags_type : enum_type);
           return NULL;
         }
 
-      g_assert (type_string == NULL);
-      type_string = "s";
+      type_string = flags_type ? "as" : "s";
+      strinfo = enum_state->strinfo;
     }
   else
     {
@@ -785,37 +920,115 @@ schema_state_add_key (SchemaState  *state,
           return NULL;
         }
 
-      enum_data = NULL;
+      strinfo = NULL;
     }
 
-  key = key_state_new (type_string, state->gettext_domain, enum_data);
+  key = key_state_new (type_string, state->gettext_domain,
+                       enum_type != NULL, flags_type != NULL, strinfo);
   g_hash_table_insert (state->keys, g_strdup (name), key);
 
   return key;
 }
 
+static void
+schema_state_add_override (SchemaState  *state,
+                           KeyState    **key_state,
+                           GString     **string,
+                           const gchar  *key,
+                           const gchar  *l10n,
+                           const gchar  *context,
+                           GError      **error)
+{
+  SchemaState *parent;
+  KeyState *original;
+
+  if (state->extends == NULL)
+    {
+      g_set_error_literal (error, G_MARKUP_ERROR,
+                           G_MARKUP_ERROR_INVALID_CONTENT,
+                           "<override> given but schema isn't "
+                           "extending anything");
+      return;
+    }
+
+  for (parent = state->extends; parent; parent = parent->extends)
+    if ((original = g_hash_table_lookup (parent->keys, key)))
+      break;
+
+  if (original == NULL)
+    {
+      g_set_error (error, G_MARKUP_ERROR,
+                   G_MARKUP_ERROR_INVALID_CONTENT,
+                   "no <key name='%s'> to override", key);
+      return;
+    }
+
+  if (g_hash_table_lookup (state->keys, key))
+    {
+      g_set_error (error, G_MARKUP_ERROR,
+                   G_MARKUP_ERROR_INVALID_CONTENT,
+                   "<override name='%s'> already specified", key);
+      return;
+    }
+
+  *key_state = key_state_override (original, state->gettext_domain);
+  *string = key_state_start_default (*key_state, l10n, context, error);
+  g_hash_table_insert (state->keys, g_strdup (key), *key_state);
+}
+
+static void
+override_state_end (KeyState **key_state,
+                    GString  **string,
+                    GError   **error)
+{
+  key_state_end_default (*key_state, string, error);
+  *key_state = NULL;
+}
+
 /* Handling of toplevel state {{{1 */
 typedef struct
 {
   GHashTable  *schema_table;            /* string -> SchemaState */
-  GHashTable  *enum_table;              /* string -> GString */
+  GHashTable  *flags_table;             /* string -> EnumState */
+  GHashTable  *enum_table;              /* string -> EnumState */
 
   gchar       *schemalist_domain;       /* the <schemalist> gettext domain */
 
   SchemaState *schema_state;            /* non-NULL when inside <schema> */
   KeyState    *key_state;               /* non-NULL when inside <key> */
-  GString     *enum_state;              /* non-NULL when inside <enum> */
+  EnumState   *enum_state;              /* non-NULL when inside <enum> */
 
   GString     *string;                  /* non-NULL when accepting text */
 } ParseState;
 
+static gboolean
+is_subclass (const gchar *class_name,
+             const gchar *possible_parent,
+             GHashTable  *schema_table)
+{
+  SchemaState *class;
+
+  if (strcmp (class_name, possible_parent) == 0)
+    return TRUE;
+
+  class = g_hash_table_lookup (schema_table, class_name);
+  g_assert (class != NULL);
+
+  return class->extends_name &&
+         is_subclass (class->extends_name, possible_parent, schema_table);
+}
+
 static void
 parse_state_start_schema (ParseState  *state,
                           const gchar  *id,
                           const gchar  *path,
                           const gchar  *gettext_domain,
+                          const gchar  *extends_name,
+                          const gchar  *list_of,
                           GError      **error)
 {
+  SchemaState *extends;
+
   if (g_hash_table_lookup (state->schema_table, id))
     {
       g_set_error (error, G_MARKUP_ERROR,
@@ -824,6 +1037,66 @@ parse_state_start_schema (ParseState  *state,
       return;
     }
 
+  if (extends_name)
+    {
+      extends = g_hash_table_lookup (state->schema_table, extends_name);
+
+      if (extends == NULL)
+        {
+          g_set_error (error, G_MARKUP_ERROR,
+                       G_MARKUP_ERROR_INVALID_CONTENT,
+                       "<schema id='%s'> extends not yet "
+                       "existing schema '%s'", id, extends_name);
+          return;
+        }
+    }
+  else
+    extends = NULL;
+
+  if (list_of)
+    {
+      if (!g_hash_table_lookup (state->schema_table, list_of))
+        {
+          g_set_error (error, G_MARKUP_ERROR,
+                       G_MARKUP_ERROR_INVALID_CONTENT,
+                       "<schema id='%s'> is list of not yet "
+                       "existing schema '%s'", id, list_of);
+          return;
+        }
+    }
+
+  if (extends)
+    {
+      if (list_of)
+        {
+          if (extends->list_of == NULL)
+            {
+              g_set_error (error, G_MARKUP_ERROR,
+                           G_MARKUP_ERROR_INVALID_CONTENT,
+                           "<schema id='%s'> is a list, extending "
+                           "<schema id='%s'> which is not a list",
+                           id, extends_name);
+              return;
+            }
+
+          if (!is_subclass (list_of, extends->list_of, state->schema_table))
+            {
+              g_set_error (error, G_MARKUP_ERROR,
+                           G_MARKUP_ERROR_INVALID_CONTENT,
+                           "<schema id='%s' list-of='%s'> extends <schema "
+                           "id='%s' list-of='%s'> but '%s' does not extend '%s'",
+                           id, list_of, extends_name, extends->list_of,
+                           list_of, extends->list_of);
+              return;
+            }
+        }
+      else
+        /* by default we are a list of the same thing that the schema
+         * we are extending is a list of (which might be nothing)
+         */
+        list_of = extends->list_of;
+    }
+
   if (path && !(g_str_has_prefix (path, "/") && g_str_has_suffix (path, "/")))
     {
       g_set_error (error, G_MARKUP_ERROR,
@@ -833,7 +1106,8 @@ parse_state_start_schema (ParseState  *state,
       return;
     }
 
-  state->schema_state = schema_state_new (path, gettext_domain);
+  state->schema_state = schema_state_new (path, gettext_domain,
+                                          extends, extends_name, list_of);
   g_hash_table_insert (state->schema_table, g_strdup (id),
                        state->schema_state);
 }
@@ -841,18 +1115,22 @@ parse_state_start_schema (ParseState  *state,
 static void
 parse_state_start_enum (ParseState   *state,
                         const gchar  *id,
+                        gboolean      is_flags,
                         GError      **error)
 {
-  if (g_hash_table_lookup (state->enum_table, id))
+  GHashTable *table = is_flags ? state->flags_table : state->enum_table;
+
+  if (g_hash_table_lookup (table, id))
     {
       g_set_error (error, G_MARKUP_ERROR,
                    G_MARKUP_ERROR_INVALID_CONTENT,
-                   "<enum id='%s'> already specified", id);
+                   "<%s id='%s'> already specified",
+                   is_flags ? "flags" : "enum", id);
       return;
     }
 
-  state->enum_state = g_string_new (NULL);
-  g_hash_table_insert (state->enum_table, g_strdup (id), state->enum_state);
+  state->enum_state = enum_state_new (is_flags);
+  g_hash_table_insert (table, g_strdup (id), state->enum_state);
 }
 
 /* GMarkup Parser Functions {{{1 */
@@ -900,11 +1178,14 @@ start_element (GMarkupParseContext  *context,
     {
       if (strcmp (element_name, "schema") == 0)
         {
-          const gchar *id, *path, *gettext_domain;
+          const gchar *id, *path, *gettext_domain, *extends, *list_of;
           if (COLLECT (STRING, "id", &id,
                        OPTIONAL | STRING, "path", &path,
-                       OPTIONAL | STRDUP, "gettext-domain", &gettext_domain))
-            parse_state_start_schema (state, id, path, gettext_domain, error);
+                       OPTIONAL | STRING, "gettext-domain", &gettext_domain,
+                       OPTIONAL | STRING, "extends", &extends,
+                       OPTIONAL | STRING, "list-of", &list_of))
+            parse_state_start_schema (state, id, path, gettext_domain,
+                                      extends, list_of, error);
           return;
         }
 
@@ -912,7 +1193,15 @@ start_element (GMarkupParseContext  *context,
         {
           const gchar *id;
           if (COLLECT (STRING, "id", &id))
-            parse_state_start_enum (state, id, error);
+            parse_state_start_enum (state, id, FALSE, error);
+          return;
+        }
+
+      else if (strcmp (element_name, "flags") == 0)
+        {
+          const gchar *id;
+          if (COLLECT (STRING, "id", &id))
+            parse_state_start_enum (state, id, TRUE, error);
           return;
         }
     }
@@ -923,16 +1212,19 @@ start_element (GMarkupParseContext  *context,
     {
       if (strcmp (element_name, "key") == 0)
         {
-          const gchar *name, *type_string, *enum_type;
+          const gchar *name, *type_string, *enum_type, *flags_type;
 
-          if (COLLECT (STRING,            "name", &name,
-                       OPTIONAL | STRING, "type", &type_string,
-                       OPTIONAL | STRING, "enum", &enum_type))
+          if (COLLECT (STRING,            "name",  &name,
+                       OPTIONAL | STRING, "type",  &type_string,
+                       OPTIONAL | STRING, "enum",  &enum_type,
+                       OPTIONAL | STRING, "flags", &flags_type))
 
             state->key_state = schema_state_add_key (state->schema_state,
                                                      state->enum_table,
+                                                     state->flags_table,
                                                      name, type_string,
-                                                     enum_type, error);
+                                                     enum_type, flags_type,
+                                                     error);
           return;
         }
       else if (strcmp (element_name, "child") == 0)
@@ -944,8 +1236,19 @@ start_element (GMarkupParseContext  *context,
                                     name, schema, error);
           return;
         }
-    }
+      else if (strcmp (element_name, "override") == 0)
+        {
+          const gchar *name, *l10n, *context;
 
+          if (COLLECT (STRING,            "name",    &name,
+                       OPTIONAL | STRING, "l10n",    &l10n,
+                       OPTIONAL | STRING, "context", &context))
+            schema_state_add_override (state->schema_state,
+                                       &state->key_state, &state->string,
+                                       name, l10n, context, error);
+          return;
+        }
+    }
 
   /* children of <key> {{{3 */
   else if (strcmp (container, "key") == 0)
@@ -1019,7 +1322,8 @@ start_element (GMarkupParseContext  *context,
 
 
   /* children of <enum> {{{3 */
-  else if (strcmp (container, "enum") == 0)
+  else if (strcmp (container, "enum") == 0 ||
+           strcmp (container, "flags") == 0)
     {
       if (strcmp (element_name, "value") == 0)
         {
@@ -1085,12 +1389,16 @@ end_element (GMarkupParseContext  *context,
       state->schemalist_domain = NULL;
     }
 
-  else if (strcmp (element_name, "enum") == 0)
+  else if (strcmp (element_name, "enum") == 0 ||
+           strcmp (element_name, "flags") == 0)
     enum_state_end (&state->enum_state, error);
 
   else if (strcmp (element_name, "schema") == 0)
     schema_state_end (&state->schema_state, error);
 
+  else if (strcmp (element_name, "override") == 0)
+    override_state_end (&state->key_state, &state->string, error);
+
   else if (strcmp (element_name, "key") == 0)
     key_state_end (&state->key_state, error);
 
@@ -1204,6 +1512,14 @@ output_schema (gpointer key,
   if (state->path)
     gvdb_hash_table_insert_string (data.pair.table, ".path", state->path);
 
+  if (state->extends_name)
+    gvdb_hash_table_insert_string (data.pair.table, ".extends",
+                                   state->extends_name);
+
+  if (state->list_of)
+    gvdb_hash_table_insert_string (data.pair.table, ".list-of",
+                                   state->extends_name);
+
   if (data.l10n)
     gvdb_hash_table_insert_string (data.pair.table,
                                    ".gettext-domain",
@@ -1236,39 +1552,44 @@ parse_gschema_files (gchar  **files,
                      GError **error)
 {
   GMarkupParser parser = { start_element, end_element, text };
-  GMarkupParseContext *context;
   ParseState state = { 0, };
   const gchar *filename;
 
-  context = g_markup_parse_context_new (&parser,
-                                        G_MARKUP_PREFIX_ERROR_POSITION,
-                                        &state, NULL);
-
   state.enum_table = g_hash_table_new_full (g_str_hash, g_str_equal,
                                             g_free, enum_state_free);
 
+  state.flags_table = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                             g_free, enum_state_free);
+
   state.schema_table = g_hash_table_new_full (g_str_hash, g_str_equal,
                                               g_free, schema_state_free);
 
   while ((filename = *files++) != NULL)
     {
+      GMarkupParseContext *context;
       gchar *contents;
       gsize size;
 
+      context = g_markup_parse_context_new (&parser,
+                                            G_MARKUP_PREFIX_ERROR_POSITION,
+                                            &state, NULL);
+
       if (!g_file_get_contents (filename, &contents, &size, error))
-        return FALSE;
+        return NULL;
 
       if (!g_markup_parse_context_parse (context, contents, size, error))
         {
           g_prefix_error (error, "%s: ", filename);
-          return FALSE;
+          return NULL;
         }
 
       if (!g_markup_parse_context_end_parse (context, error))
         {
           g_prefix_error (error, "%s: ", filename);
-          return FALSE;
+          return NULL;
         }
+
+      g_markup_parse_context_free (context);
     }
 
   g_hash_table_unref (state.enum_table);
@@ -1282,8 +1603,15 @@ compare_strings (gconstpointer a,
 {
   gchar *one = *(gchar **) a;
   gchar *two = *(gchar **) b;
+  gint cmp;
 
-  return strcmp (one, two);
+  cmp = g_str_has_suffix (two, ".enums.xml") -
+        g_str_has_suffix (one, ".enums.xml");
+
+  if (!cmp)
+    cmp = strcmp (one, two);
+
+  return cmp;
 }
 
 int
@@ -1303,7 +1631,7 @@ main (int argc, char **argv)
   GOptionEntry entries[] = {
     { "targetdir", 0, 0, G_OPTION_ARG_FILENAME, &targetdir, N_("where to store the gschemas.compiled file"), N_("DIRECTORY") },
     { "dry-run", 0, 0, G_OPTION_ARG_NONE, &dry_run, N_("Do not write the gschema.compiled file"), NULL },
-    { "uninstall", 0, 0, G_OPTION_ARG_NONE, &uninstall, N_("Do not give error for empty directory"), NULL },
+    { "uninstall", 0, 0, G_OPTION_ARG_NONE, &uninstall, N_("This option will be removed soon.") },
     { "allow-any-name", 0, 0, G_OPTION_ARG_NONE, &allow_any_name, N_("Do not enforce key name restrictions") },
 
     /* These options are only for use in the gschema-compile tests */
@@ -1365,16 +1693,15 @@ main (int argc, char **argv)
 
       if (files->len == 0)
         {
-          if (uninstall)
-            {
-              g_unlink (target);
-              return 0;
-            }
+          fprintf (stderr, _("No schema files found: "));
+
+          if (g_unlink (target))
+            fprintf (stderr, _("doing nothing.\n"));
+
           else
-            {
-              fprintf (stderr, _("No schema files found\n"));
-              return 1;
-            }
+            fprintf (stderr, _("removed existing output file.\n"));
+
+          return 0;
         }
       g_ptr_array_sort (files, compare_strings);
       g_ptr_array_add (files, NULL);
