@@ -33,7 +33,6 @@
 #include <string.h>  /* memset */
 
 #include "glib.h"
-#include "galias.h"
 
 /**
  * SECTION: hash_tables
@@ -767,7 +766,7 @@ iter_remove_or_steal (RealIter *ri, gboolean notify)
 }
 
 /**
- * g_hash_table_iter_remove():
+ * g_hash_table_iter_remove:
  * @iter: an initialized #GHashTableIter.
  *
  * Removes the key/value pair currently pointed to by the iterator
@@ -789,7 +788,7 @@ g_hash_table_iter_remove (GHashTableIter *iter)
 }
 
 /**
- * g_hash_table_iter_steal():
+ * g_hash_table_iter_steal:
  * @iter: an initialized #GHashTableIter.
  *
  * Removes the key/value pair currently pointed to by the iterator
@@ -1456,6 +1455,3 @@ g_hash_table_get_values (GHashTable *hash_table)
 
   return retval;
 }
-
-#define __G_HASH_C__
-#include "galiasdef.c"
