@@ -27,12 +27,6 @@
 #include "config.h"
 
 #include "glib.h"
-#include "galias.h"
-
-
-G_LOCK_DEFINE_STATIC (queue_memchunk);
-static GMemChunk   *queue_memchunk = NULL;
-static GTrashStack *free_queue_nodes = NULL;
 
 /**
  * g_queue_new:
@@ -44,34 +38,16 @@ static GTrashStack *free_queue_nodes = NULL;
 GQueue*
 g_queue_new (void)
 {
-  GQueue *queue;
-
-  G_LOCK (queue_memchunk);
-  queue = g_trash_stack_pop (&free_queue_nodes);
-
-  if (!queue)
-    {
-      if (!queue_memchunk)
-	queue_memchunk = g_mem_chunk_new ("GLib GQueue chunk",
-					  sizeof (GQueue),
-					  sizeof (GQueue) * 128,
-					  G_ALLOC_ONLY);
-      queue = g_chunk_new (GQueue, queue_memchunk);
-    }
-  G_UNLOCK (queue_memchunk);
-
-  queue->head = NULL;
-  queue->tail = NULL;
-  queue->length = 0;
-
-  return queue;
+  return g_slice_new0 (GQueue);
 }
 
 /**
  * g_queue_free:
  * @queue: a #GQueue.
  *
- * Frees the memory allocated for the #GQueue.
+ * Frees the memory allocated for the #GQueue. Only call this function if
+ * @queue was created with g_queue_new(). If queue elements contain
+ * dynamically-allocated memory, they should be freed first.
  **/
 void
 g_queue_free (GQueue *queue)
@@ -79,15 +55,45 @@ g_queue_free (GQueue *queue)
   g_return_if_fail (queue != NULL);
 
   g_list_free (queue->head);
+  g_slice_free (GQueue, queue);
+}
 
-#ifdef ENABLE_GC_FRIENDLY
-  queue->head = NULL;
-  queue->tail = NULL;
-#endif /* ENABLE_GC_FRIENDLY */
+/**
+ * g_queue_init:
+ * @queue: an uninitialized #GQueue
+ *
+ * A statically-allocated #GQueue must be initialized with this function
+ * before it can be used. Alternatively you can initialize it with
+ * #G_QUEUE_INIT. It is not necessary to initialize queues created with
+ * g_queue_new().
+ *
+ * Since: 2.14
+ **/
+void
+g_queue_init (GQueue *queue)
+{
+  g_return_if_fail (queue != NULL);
 
-  G_LOCK (queue_memchunk);
-  g_trash_stack_push (&free_queue_nodes, queue);
-  G_UNLOCK (queue_memchunk);
+  queue->head = queue->tail = NULL;
+  queue->length = 0;
+}
+
+/**
+ * g_queue_clear:
+ * @queue: a #GQueue
+ *
+ * Removes all the elements in @queue. If queue elements contain
+ * dynamically-allocated memory, they should be freed first.
+ *
+ * Since: 2.14
+ */
+void
+g_queue_clear (GQueue *queue)
+{
+  g_return_if_fail (queue != NULL);
+
+  g_list_free (queue->head);
+  g_queue_init (queue);
 }
 
 /**
@@ -975,7 +981,7 @@ g_queue_insert_after (GQueue   *queue,
  *     called with two elements of the @queue and @user_data. It should
  *     return 0 if the elements are equal, a negative value if the first
  *     element comes before the second, and a positive value if the second
- *     element comes after the first.
+ *     element comes before the first.
  * @user_data: user data passed to @func.
  * 
  * Inserts @data into @queue using @func to determine the new position.
@@ -1001,6 +1007,3 @@ g_queue_insert_sorted (GQueue           *queue,
   else
     g_queue_push_tail (queue, data);
 }
-
-#define __G_QUEUE_C__
-#include "galiasdef.c"
