@@ -38,7 +38,6 @@
 #define PCRE_ERROR_NULLWSLIMIT (-22)
 #endif
 
-#include "galias.h"
 
 /* Mask of all the possible values for GRegexCompileFlags. */
 #define G_REGEX_COMPILE_MASK (G_REGEX_CASELESS		| \
@@ -498,7 +497,6 @@ gboolean
 g_match_info_next (GMatchInfo  *match_info,
 		   GError     **error)
 {
-  gint opts;
   gint prev_match_start;
   gint prev_match_end;
 
@@ -506,8 +504,6 @@ g_match_info_next (GMatchInfo  *match_info,
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
   g_return_val_if_fail (match_info->pos >= 0, FALSE);
 
-  opts = match_info->regex->match_opts | match_info->match_opts;
- 
   prev_match_start = match_info->offsets[0];
   prev_match_end = match_info->offsets[1];
 
@@ -516,8 +512,7 @@ g_match_info_next (GMatchInfo  *match_info,
 				   match_info->string,
 				   match_info->string_len,
 				   match_info->pos,
-				   match_info->regex->match_opts |
-				   match_info->match_opts,
+				   match_info->regex->match_opts | match_info->match_opts,
 				   match_info->offsets,
                                    match_info->n_offsets);
   if (IS_PCRE_ERROR (match_info->matches))
@@ -1271,6 +1266,42 @@ g_regex_get_capture_count (const GRegex *regex)
   return value;
 }
 
+/**
+ * g_regex_get_compile_flags:
+ * @regex: a #GRegex
+ *
+ * Returns the compile options that @regex was created with.
+ *
+ * Returns: flags from #GRegexCompileFlags
+ *
+ * Since: 2.26
+ */
+GRegexCompileFlags
+g_regex_get_compile_flags (GRegex *regex)
+{
+  g_return_val_if_fail (regex != NULL, 0);
+
+  return regex->compile_opts;
+}
+
+/**
+ * g_regex_get_match_flags:
+ * @regex: a #GRegex
+ *
+ * Returns the match options that @regex was created with.
+ *
+ * Returns: flags from #GRegexMatchFlags
+ *
+ * Since: 2.26
+ */
+GRegexMatchFlags
+g_regex_get_match_flags (GRegex *regex)
+{
+  g_return_val_if_fail (regex != NULL, 0);
+
+  return regex->match_opts;
+}
+
 /**
  * g_regex_match_simple:
  * @pattern: the regular expression
@@ -2757,6 +2788,3 @@ g_regex_escape_string (const gchar *string,
 
   return g_string_free (escaped, FALSE);
 }
-
-#define __G_REGEX_C__
-#include "galiasdef.c"
