@@ -48,7 +48,6 @@
 #include "gprintfint.h"
 #include "glibintl.h"
 
-#include "galias.h"
 
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -277,12 +276,15 @@ g_strdup_printf (const gchar *format,
  * g_strconcat:
  * @string1: the first string to add, which must not be %NULL
  * @Varargs: a %NULL-terminated list of strings to append to the string
- * 
+ *
  * Concatenates all of the given strings into one long string.
  * The returned string should be freed with g_free() when no longer needed.
  *
+ * Note that this function is usually not the right function to use to
+ * assemble a translated message from pieces, since proper translation
+ * often requires the pieces to be reordered.
  *
- * <warning><para>The variable argument list <emphasis>must</emphasis> end 
+ * <warning><para>The variable argument list <emphasis>must</emphasis> end
  * with %NULL. If you forget the %NULL, g_strconcat() will start appending
  * random memory junk to your string.</para></warning>
  *
@@ -3270,7 +3272,3 @@ g_dngettext (const gchar *domain,
 
   return dngettext (domain, msgid, msgid_plural, n);
 }
-
-
-#define __G_STRFUNCS_C__
-#include "galiasdef.c"
