@@ -40,7 +40,7 @@
 
 #include "config.h"
 
-#include "glib.h"
+#include "gthread.h"
 #include "gthreadprivate.h"
 
 #ifdef HAVE_UNISTD_H
@@ -56,7 +56,11 @@
 
 #include <string.h>
 
-#include "galias.h"
+#include "garray.h"
+#include "gslist.h"
+#include "gtestutils.h"
+#include "gtimer.h"
+
 
 /**
  * SECTION: threads
@@ -2532,7 +2536,3 @@ g_thread_get_initialized ()
 {
   return g_thread_supported ();
 }
-
-#define __G_THREAD_C__
-#include "galiasdef.c"
-
